/*
 * BinCAS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Nov-06
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * CA MEASURE OF ASSOCIATION
 * 
 * <br>http://www.psych.cornell.edu/Darlington/crosstab/table3.htm
 * <br>CA (for "conditional association") is a modification of OGE that in 
 * principle can be defined for tables of any size, but in practice is computed 
 * easily only for 2x2 tables, so we explain it in those terms. CA deals with 
 * the fact that phi and OGE may be low partly because of the difference between 
 * row and column marginal totals. The larger this difference, the lower phi and 
 * OGE will usually be. CA is independent of the difference in marginal totals, 
 * and can be 1 regardless of the size of that difference.
 * <br>
 * <br>In a 2x2 table exhibiting some association, two diagonally opposite cells 
 * have o > e while the other two have o<e. Call these two diagonals the "major" 
 * and "minor" diagonals respectively. Let o-major denote the total observed 
 * frequency in the major diagonal, while e-major denotes the total expected 
 * frequency in those two cells as calculated by the usual formula for e. Let 
 * m denote the smaller of the two observed cell frequencies in the minor 
 * diagonal. Then it can be shown that (o-major + 2m) is the largest value 
 * that o-major could have without changing any of the marginal totals. Therefore: 
 * <br>
 * <br>CA = (o-major - e-major)/(o-major - e-major + 2m)
 * <br>
 * <br>is the amount by which the observed frequency in the major diagonal 
 * exceeds its expected value, expressed as a proportion of the largest value 
 * that difference could be given the observed marginal totals. It can be shown 
 * that CA = OGE if the column marginal total for each entry in the major 
 * diagonal equals the row marginal total for that same entry. The larger the 
 * difference between these row and column marginal totals, the more CA exceeds OGE.
 * <br>
 * <br>We can also give CA a somewhat different interpretation. Define the 
 * "extreme pattern" as the set of four cell frequencies observed when the cells 
 * in the major diagonal contain their maximum possible frequencies under the 
 * current marginals; at least one of the other two cells will then contain zero. 
 * Then pick any one of the four cells and compute (observed-null)/(extreme-null), 
 * where these values are defined as the values in that cell under the observed, 
 * null, and extreme patterns. It turns out that this procedure yields the same 
 * value as CA, regardless of what cell you picked. Therefore CA can also be 
 * described as the difference between the observed and null frequencies in any 
 * cell, expressed as a proportion of the largest possible difference in the 
 * same direction, given the observed marginal totals.
 * <br>
 * <br>Similarity [0, 1].
 * <br>Usage: for binary data.
 * 
 * @author  Martin Hassel
 * @version 2004-Nov-06
 */
public class BinCAS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		int a = 0; /* present in both samples */
		int b = 0; /* present in sample i, absent in sample j */
		int c = 0; /* present in sample j, absent in sample i */
		int d = 0; /* absent in both samples */
		double ea = 0; /* expected to be present in both samples */
		double ed = 0; /* expected to be absent in both samples */
		double omaj; /* presences observed in major diagonal */
		double emaj; /* presences expected in major diagonal */
		int m; /* smaller of the two observed cell frequencies in the minor diagonal */
		double sim_matrix[][]; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		/* Begin calculations */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				a = b = c = d = 0;
				for(int column = 0; column < columns; ++column) {
					if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 1)
						a++;
					else if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 0)
						b++;
					else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 1)
						c++;
					else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 0)
						d++;
				}
				ea = (a+b)*(a+c)/(double)columns;
				ed = (c+d)*(b+d)/(double)columns;
				omaj = a + d;
				emaj = ea + ed;
				m = b<c ? b : c;
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
					(omaj - emaj)/(double)(omaj - emaj + 2*m);
			}
		}
		return sim_matrix;
	}		
}
