/*
 * BinOGES_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Oct-25
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * OGE MEASURE OF ASSOCIATION
 * 
 * <br>http://www.psych.cornell.edu/Darlington/crosstab/table3.htm
 * <br>OGE has a frequency interpretation that many users will find more 
 * appealing than the ratio-scale interpretation of phi2. OGE stands for 
 * "o greater than e", and is based on the number of cases falling in cells 
 * for which o>e. 
 * <br>Similarity [0, 1].
 * <br>Usage: for binary data.
 * 
 * @author  Martin Hassel
 * @version 2004-Oct-25
 */
public class BinOGES_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		int a = 0; /* present in both samples */
		int b = 0; /* present in sample i, absent in sample j */
		int c = 0; /* present in sample j, absent in sample i */
		int d = 0; /* absent in both samples */
		double ea = 0; /* expected to be present in both samples */
		double eb = 0; /* expected to be present in sample i, absent in sample j */
		double ec = 0; /* expected to be present in sample j, absent in sample i */
		double ed = 0; /* expected to be absent in both samples */
		int max, observed, nil = 0; /* observed and expected frequencies */
		double sim_matrix[][]; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		  
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		  
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
		    sim_matrix[i][i] = 1.0;
		  
		/* Begin calculations */
		max = columns;
		for(int row1 = 0; row1 < rows; ++row1) {
		    for(int row2 = (row1 + 1); row2 < rows; ++row2) {
		      a = b = c = d = 0;
		      for(int column = 0; column < columns; ++column) {
				if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 1)
				  a++;
				else if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 0)
				  b++;
				else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 1)
				  c++;
				else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 0)
				  d++;
		      }
		      ea = (a+b)*(a+c)/(double)columns;
		      eb = (a+b)*(b+d)/(double)columns;
		      ec = (c+d)*(a+c)/(double)columns;
		      ed = (c+d)*(b+d)/(double)columns;
		      observed = 0;
		      if (a > ea) {
				observed += a; nil += ea;
		      }
		      if (b > eb) {
				observed += b; nil += eb;
		      }
		      if (c > ec) {
				observed += c; nil += ec;
		      }
		      if (d > ed) {
				observed += d; nil += ed;
		      }
		      sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
		      	(observed-nil)/(double)(max-nil);
		    }
		}
		return sim_matrix;
	}
}
