/*
 * BinSokalSneath1S_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Oct-02
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * KENDALL TAU RANK CORRELATION COEFFICIENT 
 * 
 * <br>http://www.psych.cornell.edu/Darlington/crosstab/table5.htm
 * <br>Similarity [-1, 1]. No ties! 
 * <br>Usage: for binary data.
 * 
 * @author  Martin Hassel
 * @version 2004-Oct-02
 */
public class KendallTauS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		int count;
		int con, dis; /* number of concordances, and discordances, respectively */
		double sim_matrix[][]; /* matrix to contain the similarity */
		int rank_matrix[][];   /* matrix to contain the ranks */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate ranks matrix */
		rank_matrix = new int[rows][columns];
		
		/* calculate ranks */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = 0; row2 < columns; ++row2) {
				/* count scores lower than this */
				count = 0;
				for(int column = 0; column < columns; ++column) {
					if(data_matrix[row1][column] <= data_matrix[row1][row2])
						count++;
				}
				rank_matrix[row1][row2]= count;
			}
		}
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		/* Begin calculations */
		con = dis = 0;
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				for(int column1 = 0; column1 < columns; ++column1) {
					for(int column2 = column1+1; column2 < columns; ++column2) {
						if(((rank_matrix[row1][column1] < rank_matrix[row1][column2]) && (rank_matrix[row2][column1] < rank_matrix[row2][column2])) || 
								((rank_matrix[row1][column1] > rank_matrix[row1][column2]) && (rank_matrix[row2][column1] > rank_matrix[row2][column2])))
							con++; 
						else if(((rank_matrix[row1][column1] < rank_matrix[row1][column2]) && (rank_matrix[row2][column1] > rank_matrix[row2][column2])) || 
								((rank_matrix[row1][column1] > rank_matrix[row1][column2]) && (rank_matrix[row2][column1] < rank_matrix[row2][column2])))
							dis++;
					}
				}
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
					(con-dis)/(double)(columns*columns/2.0-columns);
			}
		}
		return sim_matrix;
	}
}
