/*
 * MinkowskiD_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-15
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * MINKOWSKI DISTANCE
 * 
 * <br>http://disney.ctr.columbia.edu/jrsthesis/node66.html#SECTION00731000000000000000
 * 
 * @author  Martin Hassel
 * @version 2004-Sep-15
 */
public class MinkowskiD_row extends SimilarityMeasure {
	protected int _power = 2;
	
	/**
	 * Minkowski metrics are a family of distance measurements which are 
	 * generalized from the Euclidean distance formula.
	 * Two values of <code>power</code> have easily interpretable meanings: 
	 * At <code>power=2</code>, you get the typical Euclidean distance and at 
	 * <code>power=1</code> yields the city-block metric, so named because it 
	 * gives the distance you'd have to travel to get between two points if 
	 * you were walking on a grid of city streets (no real diagonals).
	 * <br>Default is <code>power=2</code>.
	 * @param power the Minkowski parameter
	 */
	public MinkowskiD_row(int power) {
		_power = power;
	}
	
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double x, y, sum;
		double dist_matrix[][]; /* matrix to contain the distance */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate distance matrix */
		dist_matrix = new double[rows][rows];
		
		/* Set diagonal to 0 */
		for(int i = 0; i < rows; ++i)
			dist_matrix[i][i] = 0.0;
		
		/* Begin calculations */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				sum = 0;
				for(int column = 0; column < columns; ++column) {
					x = data_matrix[row1][column];
					y = data_matrix[row2][column];
					sum += Math.pow(Math.abs(x-y), _power);
				}
				dist_matrix[row1][row2] = dist_matrix[row2][row1] = 
					Math.pow(sum, 1/(double)_power);
			}
		}
		return dist_matrix;		
	}
}
